from __future__ import annotations

from typing import Optional

from digitalhub.entities.task._base.models import CoreServiceType
from digitalhub.entities.task._base.spec import TaskSpecFunction, TaskValidatorFunction
from pydantic import Field


class TaskSpecModelserveServe(TaskSpecFunction):
    """TaskSpecModelserveServe specifications."""

    def __init__(
        self,
        function: str,
        node_selector: list[dict] | None = None,
        volumes: list[dict] | None = None,
        resources: dict | None = None,
        affinity: dict | None = None,
        tolerations: list[dict] | None = None,
        envs: list[dict] | None = None,
        secrets: list[str] | None = None,
        profile: str | None = None,
        runtime_class: str | None = None,
        priority_class: str | None = None,
        replicas: int | None = None,
        service_type: str | None = None,
        **kwargs,
    ) -> None:
        super().__init__(
            function,
            node_selector,
            volumes,
            resources,
            affinity,
            tolerations,
            envs,
            secrets,
            profile,
            runtime_class,
            priority_class,
            **kwargs,
        )
        self.replicas = replicas
        self.service_type = service_type


class TaskValidatorModelserveServe(TaskValidatorFunction):
    """
    TaskValidatorModelserveServe specifications.
    """

    replicas: Optional[int] = Field(default=None, ge=1)
    """Number of replicas."""

    service_type: CoreServiceType = Field(default=CoreServiceType.NODE_PORT.value)
    """Service type."""
